@@ -1,36 +1,33 @@
-#' Gets transcript fragments in a given format.
+#' Gets transcript fragments in a given format
 #' 
 #' This function gets fragments of transcripts from 'LaBB-CAT', 
 #' converted to a given format (by default, Praat TextGrid).
 #'
-#' \emph{NB} Although many formats will generate exactly one file for each interval
-#'      (e.g. mime.type=text/praat-textgrid), this is not guaranted; some formats generate
+#' **NB** Although many formats will generate exactly one file for each interval
+#'      (e.g. mime.type=text/praat-textgrid), this is not guaranteed; some formats generate
 #'      a single file or a fixed collection of files regardless of how many fragments there are.
 #'
 #' @param labbcat.url URL to the LaBB-CAT instance
 #' @param id The transcript ID (transcript name) of the sound recording, or
-#'     a vector of transcript IDs. 
+#'   a vector of transcript IDs. 
 #' @param start The start time in seconds, or a vector of start times.
 #' @param end The end time in seconds, or a vector of end times.
 #' @param layer.ids A vector of layer IDs.
 #' @param mime.type Optional content-type - "text/praat-textgrid" is the default, but your
-#'     LaBB-CAT installation may support other formats, which can be discovered using
-#'     \link{getSerializerDescriptors}.
+#'   LaBB-CAT installation may support other formats, which can be discovered using
+#'   [getSerializerDescriptors].
 #' @param path Optional path to directory where the files should be saved.
 #' @return The name of the file, which is saved in the current
-#'     directory, or a list of names of files, if multiple
-#'     id's/start's/end's were specified 
+#'   directory, or a list of names of files, if multiple
+#'   id's/start's/end's were specified 
 #'
-#' If a list of files is returned, they are in the order that they
-#'     were returned by the server, which *should* be the order that
-#'     they were specified in the id/start/end lists.
+#'   If a list of files is returned, they are in the order that they
+#'   were returned by the server, which *should* be the order that
+#'   they were specified in the id/start/end lists.
 #' 
-#' @seealso \link{getSerializerDescriptors}
+#' @seealso [getSerializerDescriptors]
 #' @examples
 #' \dontrun{
-#' ## define the LaBB-CAT URL
-#' labbcat.url <- "https://labbcat.canterbury.ac.nz/demo/"
-#' 
 #' ## Get the 5 seconds starting from 10s after the beginning of a recording
 #' textgrid.file <- getFragments(labbcat.url, "AP2505_Nelson.eaf", 10.0, 15.0,
 #'     c("transcript", "phonemes"), path="samples") 
@@ -105,12 +102,12 @@ getFragments <- function(labbcat.url, id, start, end, layer.ids, mime.type = "te
             file.remove(file.name)
             file.name <<- NULL
         } else {
-            content.disposition <- as.character(httr::headers(resp)["content-disposition"])
-            content.disposition.parts <- strsplit(content.disposition, "=")
-            if (length(content.disposition.parts[[1]]) > 1
-                && file.name != content.disposition.parts[[1]][2]) {
+            content.disposition.filename <- fileNameFromContentDisposition(
+                as.character(httr::headers(resp)["content-disposition"]))
+            if (!is.null(content.disposition.filename)
+                && file.name != content.disposition.filename) {
                 ## file name is specified, so use it
-                final.file.name <- paste(dir, content.disposition.parts[[1]][2], sep="")
+                final.file.name <- paste(dir, content.disposition.filename, sep="")
                 file.rename(file.name, final.file.name)
                 file.name <- final.file.name
             }
@@ -126,12 +123,7 @@ getFragments <- function(labbcat.url, id, start, end, layer.ids, mime.type = "te
                 file.remove(file.name)
             } else { ## a single file returned
                 ## move it to the dir
-                if (stringr::str_length(dir) > 0) { ## directory is specified
-                    file.names <- paste(dir, .Platform$file.sep, file.name, sep="")
-                    file.rename(file.name, file.names)
-                } else {
-                    file.names = file.name
-                }
+                file.names = file.name
             }
         }
     }, error = function(e) {
